#!/usr/bin/env python3
# Adapted from custom_dsstore.py by the Jalview team 2024
# Copyright (c) 2013-2017 The Bitcoin Core developers
# Distributed under the MIT software license, see the accompanying
# file COPYING or http://www.opensource.org/licenses/mit-license.php.
from ds_store import DSStore
from pprint import pprint
import mac_alias
import argparse
import json

parser = argparse.ArgumentParser(
  prog = "jalview_custom_dsstore.py",
  description = "Take an existing DS_Store file and change the volume name and add path to background image"
  )
parser.add_argument("-i", "--input", help="The existing styled DS_Store file", dest="input")
parser.add_argument("-o", "--output", help="The output DS_Store file", dest="output")
parser.add_argument("-v", "--volumename", help="The name of the output DS_Store volume", dest="volumename")
parser.add_argument("-b", "--background", help="The background image filename to use", dest="background")
parser.add_argument("-c", "--config", help="YAML configuration for window and icon sizes and positions", dest="config")
parser.add_argument("-d", "--dump", help="Display contents of the input DS_Store to stdout", action="store_true", dest="dump")
parser.add_argument("-q", "--quiet", help="Don't print messages to stdout", action="store_true", dest="quiet")
parser.add_argument("-f", "--filenames", help="List of comma-separated filenames to show information for --dump", dest="filenames", default="")

args = parser.parse_args()

if (args.dump and not args.input):
  exit("When --dump used, an --input must be given")

if (args.dump):
  with DSStore.open(args.input, 'r+') as d:
    for key in ["bwsp", "icvp","Iloc"]:
      try:
        myprint(f"d['.']['{key}']=")
        mypprint(str(d['.'][key]))
      except:
        myprint(f"No info for d['.']['{key}']")

    for data in d:
      try:
        data = str(data)
        mypprint(f"d['{data}']=",d[data])
      except:
        myprint(f"No info for d['{data}']")
#    for filename in args.filenames.split(","):
#      if (filename):
#        for f in d.find(filename):
#          myprint(f"d['{f}']=", d[f])
#        else:
#          myprint(f"# No filename '{filename}' found in DS_Store '{args.input}'")
    exit(0)

if (args.output and not (args.input or args.config)):
  exit("Need --input FILENAME or --config FILENAME to produce an --output")

if (not args.output):
  exit("Provide --output FILENAME to output DS_Store")

if (not args.volumename):
  exit("Provide a volume name with --volumename NAME")

if (not args.background):
  exit("Provide a background image filename (just the file, no path) with --background FILENAME")

package_name_ns = args.volumename
configfilename = args.config
configfile = open(configfilename, 'r')

config = json.load(configfile)
inputds = DSStore.open(args.input)
outputds = DSStore.open(args.output, 'w+')

def myprint(string):
  if (not args.quiet):
    print(string)

def mypprint(string):
  if (not args.quiet):
    pprint(string)

bwsp = {}

for key in "ShowStatusBar SidebarWidthTenElevenOrLater ShowToolbar ShowTabView ContainerShowSidebar WindowBounds ShowSidebar ShowPathbar".split():
  bwsp[key] = inputds['.']['bwsp'][key]
  myprint(f"Setting bwsp['{key}'] to '"+str(inputds['.']['bwsp'][key])+"'")

outputds['.']['bwsp'] = bwsp

icvp = {}

alias = mac_alias.Alias.from_bytes(inputds['.']['icvp']['backgroundImageAlias'])
alias.volume.name = args.volumename
alias.volume.posix_path = '/Volumes/' + args.volumename
alias.volume.disk_image_alias.target.filename = args.volumename + '.dmg'
alias.volume.disk_image_alias.target.carbon_path = 'Macintosh HD:Users:\x00Jalview:\x00' + args.volumename + ':\x00' + args.volumename + '.dmg'
alias.volume.disk_image_alias.target.posix_path = '/Users/Jalview/' + args.volumename + '.dmg'
alias.target.carbon_path = args.volumename + ':.background:\x00' + args.background

icvp['backgroundImageAlias'] = alias.to_bytes()

for key in "backgroundColorRed backgroundColorBlue backgroundColorGreen gridSpacing gridOffsetX gridOffsetY showItemInfo viewOptionsVersion arrangeBy textSize labelOnBottom backgroundType showIconPreview iconSize".split():
  icvp[key] = inputds['.']['icvp'][key]
  myprint(f"Setting icvp['{key}'] to '"+str(inputds['.']['icvp'][key])+"'")

outputds['.']['icvp'] = icvp

# copy filenames properties? not working, using JSON config file
#for filename in args.filenames.split(","):
#  outputds[filename]['Iloc'] = inputds[filename]['Iloc']

outputds['.']['vSrn'] = ('long', 1)

for fileinfo in config['files']:
  outputds[fileinfo['name']]['Iloc'] = (fileinfo['xpos'], fileinfo['ypos'])

outputds.flush()
outputds.close()
inputds.close()

